@@ -481,10 +481,20 @@ REGISTER_FUNCTION(logb, eval_func(logb_eval).
 
 static ex Li2_evalf(const ex & x, PyObject* parent)
 {
-	if (is_exactly_a<numeric>(x))
-		return Li2(ex_to<numeric>(x), parent);
-	
-	return Li2(x).hold();
+	if (not is_exactly_a<numeric>(x))
+	        return Li2(x).hold();
+
+        const numeric& num = ex_to<numeric>(x);
+        try {
+                return numeric::try_py_method(num, "dilog");
+        }
+        catch (std::logic_error) {}
+        try {
+                return numeric::try_py_method(num, "polylog", *_num2_p);
+        }
+        catch (std::logic_error) {}
+
+        return Li2(num, parent);
 }
 
 static ex Li2_eval(const ex & x)
@@ -638,10 +648,19 @@ unsigned Li2_SERIAL::serial = function::register_new(function_options("dilog", 1
 
 static ex Li_evalf(const ex& m_, const ex& x_, PyObject* parent)
 {
-	if (is_exactly_a<numeric>(m_) and is_exactly_a<numeric>(x_))
-                return Li2(ex_to<numeric>(m_), ex_to<numeric>(x_), parent);
+	if (not is_exactly_a<numeric>(m_)
+            or not is_exactly_a<numeric>(x_))
+                return Li(m_,x_).hold();
 
-        return Li(m_,x_).hold();
+        const numeric& num_m = ex_to<numeric>(m_);
+        const numeric& num_x = ex_to<numeric>(x_);
+        
+        try {
+                return numeric::try_py_method(num_x, "polylog", num_m);
+        }
+        catch (std::logic_error) {}
+
+        return Li2(num_m, num_x, parent);
 }
 
 
